@@ -16,16 +16,20 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ev-document-misc.h 3095 2008-08-03 18:32:42Z wbolster $
+ *  $Id$
  */
 
+#if !defined (__EV_EVINCE_DOCUMENT_H_INSIDE__) && !defined (EVINCE_COMPILATION)
+#error "Only <evince-document.h> can be included directly."
+#endif
+
 #ifndef EV_DOCUMENT_MISC_H
 #define EV_DOCUMENT_MISC_H
 
 #include <cairo.h>
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gtk/gtkstyle.h>
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
