@@ -23,7 +23,7 @@
 
 #include "ev-document.h"
 
-#include <gtk/gtkwidget.h>
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
